@@ -203,7 +203,7 @@ return this.labels[index] || "";
         renderers = list(renderers)
 
         if is_datetime_index:
-            formatters = dict(datetime='datetime')
+            formatters = {'@datetime': 'datetime'}
             tooltips = [("Date", "@datetime{%c}")] + tooltips
         else:
             formatters = {}
@@ -459,7 +459,7 @@ return this.labels[index] || "";
                 figs_below_ohlc.append(fig)
             tooltips = []
             colors = value._opts['color']
-            colors = colors and cycle([_as_list(colors)[0]]) or (
+            colors = colors and cycle(_as_list(colors)) or (
                 cycle([next(ohlc_colors)]) if is_overlay else colorgen())
             legend_label = LegendStr(value.name)
             for j, arr in enumerate(value, 1):
@@ -542,8 +542,8 @@ return this.labels[index] || "";
     if plot_volume:
         custom_js_args.update(volume_range=fig_volume.y_range)
 
-    fig_ohlc.x_range.callback = CustomJS(args=custom_js_args,
-                                         code=_AUTOSCALE_JS_CALLBACK)
+    fig_ohlc.x_range.js_on_change('end', CustomJS(args=custom_js_args,
+                                                  code=_AUTOSCALE_JS_CALLBACK))
 
     plots = figs_above_ohlc + [fig_ohlc] + figs_below_ohlc
     for f in plots:
