@@ -26,7 +26,7 @@ except ImportError:
         return seq
 
 from ._plotting import plot
-from ._util import _as_str, _Indicator, _Data, _data_period
+from ._util import _as_str, _Indicator, _Data, _data_period, try_
 
 
 __pdoc__ = {
@@ -73,7 +73,7 @@ class Strategy(metaclass=ABCMeta):
             setattr(self, k, v)
         return params
 
-    def I(self,  # noqa: E743
+    def I(self,  # noqa: E741, E743
           func: Callable, *args,
           name=None, plot=True, overlay=None, color=None, scatter=False,
           **kwargs) -> np.ndarray:
@@ -115,24 +115,32 @@ class Strategy(metaclass=ABCMeta):
         """
         if name is None:
             params = ','.join(filter(None, map(_as_str, chain(args, kwargs.values()))))
-            func_name = func.__name__.replace('<lambda>', 'λ')
+            func_name = _as_str(func)
             name = ('{}({})' if params else '{}').format(func_name, params)
         else:
             name = name.format(*map(_as_str, args),
                                **dict(zip(kwargs.keys(), map(_as_str, kwargs.values()))))
 
-        value = func(*args, **kwargs)
-
         try:
-            if isinstance(value, pd.DataFrame):
-                value = value.values.T
-            value = np.asarray(value)
-        except Exception:
-            raise ValueError('Indicators must return array-like sequences of values')
-        if value.shape[-1] != len(self._data.Close):
-            raise ValueError('Indicators must be (a tuple of) arrays of same length as `data`'
-                             '(data: {}, indicator "{}": {})'.format(len(self._data.Close),
-                                                                     name, value.shape))
+            value = func(*args, **kwargs)
+        except Exception as e:
+            raise RuntimeError('Indicator "{}" errored with exception: {}'.format(name, e))
+
+        if isinstance(value, pd.DataFrame):
+            value = value.values.T
+
+        value = try_(lambda: np.asarray(value, order='C'), None)
+        is_arraylike = value is not None
+
+        # Optionally flip the array if the user returned e.g. `df.values`
+        if is_arraylike and np.argmax(value.shape) == 0:
+            value = value.T
+
+        if not is_arraylike or not 1 <= value.ndim <= 2 or value.shape[-1] != len(self._data.Close):
+            raise ValueError(
+                'Indicators must return (optionally a tuple of) numpy.arrays of same '
+                'length as `data`(data shape: {}; indicator "{}" shape: {}, value: {})'
+                .format(self._data.Close.shape, name, getattr(value, 'shape', ''), value))
 
         if plot and overlay is None and np.issubdtype(value.dtype, np.number):
             x = value / self._data.Close
@@ -690,7 +698,7 @@ class Backtest:
 
         # Skip first few candles where indicators are still "warming up"
         # +1 to have at least two entries available
-        start = 1 + max((np.isnan(indicator.astype(float)).argmin()
+        start = 1 + max((np.isnan(indicator.astype(float)).argmin(axis=-1).max()
                          for _, indicator in indicator_attrs), default=0)
 
         # Disable "invalid value encountered in ..." warnings. Comparison
@@ -741,7 +749,7 @@ class Backtest:
         series, an additional `pd.Series` is returned with a multiindex
         of all admissible parameter combinations, which can be further
         inspected or projected onto 2D to plot a heatmap
-        (see `backtesting.backtesting.lib.plot_heatmaps()`).
+        (see `backtesting.lib.plot_heatmaps()`).
 
         Additional keyword arguments represent strategy arguments with
         list-like collections of possible values. For example, the following
@@ -754,6 +762,9 @@ class Backtest:
         .. TODO::
             Add parameter `max_tries: Union[int, float] = None` which switches
             from exhaustive grid search to random search. See notes in the source.
+
+        .. TODO::
+            Improve multiprocessing/parallel execution on Windos with start method 'spawn'.
         """
         if not kwargs:
             raise ValueError('Need some strategy parameters to optimize')
@@ -816,23 +827,35 @@ class Backtest:
             for i in range(0, len(seq), n):
                 yield seq[i:i + n]
 
-        # If multiprocessing start method is 'fork' (i.e. on POSIX), use
-        # a pool of processes to compute results in parallel.
-        # Otherwise (i.e. on Windos), sequential computation will be "faster".
-        if mp.get_start_method(allow_none=False) == 'fork':
-            with ProcessPoolExecutor() as executor:
-                futures = [executor.submit(self._mp_task, params)
-                           for params in _batch(param_combos)]
-                for future in _tqdm(as_completed(futures), total=len(futures)):
-                    for params, stats in future.result():
-                        heatmap[tuple(params.values())] = maximize(stats)
-        else:
-            if os.name == 'posix':
-                warnings.warn("For multiprocessing support in `Backtest.optimize()` "
-                              "set multiprocessing start method to 'fork'.")
-            for params in _tqdm(param_combos):
-                for _, stats in self._mp_task([params]):
-                    heatmap[tuple(params.values())] = maximize(stats)
+        # Save necessary objects into "global" state; pass into concurrent executor
+        # (and thus pickle) nothing but two numbers; receive nothing but numbers.
+        # With start method "fork", children processes will inherit parent address space
+        # in a copy-on-write manner, achieving better performance/RAM benefit.
+        backtest_uuid = np.random.random()
+        param_batches = list(_batch(param_combos))
+        Backtest._mp_backtests[backtest_uuid] = (self, param_batches, maximize)
+        try:
+            # If multiprocessing start method is 'fork' (i.e. on POSIX), use
+            # a pool of processes to compute results in parallel.
+            # Otherwise (i.e. on Windos), sequential computation will be "faster".
+            if mp.get_start_method(allow_none=False) == 'fork':
+                with ProcessPoolExecutor() as executor:
+                    futures = [executor.submit(Backtest._mp_task, backtest_uuid, i)
+                               for i in range(len(param_batches))]
+                    for future in _tqdm(as_completed(futures), total=len(futures)):
+                        batch_index, values = future.result()
+                        for value, params in zip(values, param_batches[batch_index]):
+                            heatmap[tuple(params.values())] = value
+            else:
+                if os.name == 'posix':
+                    warnings.warn("For multiprocessing support in `Backtest.optimize()` "
+                                  "set multiprocessing start method to 'fork'.")
+                for batch_index in _tqdm(range(len(param_batches))):
+                    _, values = Backtest._mp_task(backtest_uuid, batch_index)
+                    for value, params in zip(values, param_batches[batch_index]):
+                        heatmap[tuple(params.values())] = value
+        finally:
+            del Backtest._mp_backtests[backtest_uuid]
 
         best_params = heatmap.idxmax()
 
@@ -848,10 +871,14 @@ class Backtest:
             return self._results, heatmap
         return self._results
 
-    def _mp_task(self, param_combos):
-        return [(params, stats) for params, stats in ((params, self.run(**params))
-                                                      for params in param_combos)
-                if stats['# Trades']]
+    @staticmethod
+    def _mp_task(backtest_uuid, batch_index):
+        bt, param_batches, maximize_func = Backtest._mp_backtests[backtest_uuid]
+        return batch_index, [maximize_func(stats) if stats['# Trades'] else np.nan
+                             for stats in (bt.run(**params)
+                                           for params in param_batches[batch_index])]
+
+    _mp_backtests = {}
 
     @staticmethod
     def _compute_drawdown_duration_peaks(dd: pd.Series):
@@ -918,6 +945,7 @@ class Backtest:
         s.loc['Avg. Trade [%]'] = mean_return * 100
         s.loc['Max. Trade Duration'] = _round_timedelta(durations.max())
         s.loc['Avg. Trade Duration'] = _round_timedelta(durations.mean())
+        s.loc['Profit Factor'] = returns[returns > 0].sum() / (abs(returns[returns < 0].sum()) or np.nan)  # noqa: E501
         s.loc['Expectancy [%]'] = ((returns[returns > 0].mean() * win_rate -
                                     returns[returns < 0].mean() * (100 - win_rate)))
         pl = pl.dropna()
