@@ -8,6 +8,7 @@ from glob import glob
 from runpy import run_path
 from tempfile import NamedTemporaryFile, gettempdir
 from unittest import TestCase
+from unittest.mock import patch
 
 import numpy as np
 import pandas as pd
@@ -120,6 +121,9 @@ class TestBacktest(TestCase):
                 self.sma = self.I(SMA, self.data.Close, 10)
                 self.remains_indicator = np.r_[2] * np.cumsum(self.sma * 5 + 1) * np.r_[2]
 
+                self.transpose_invalid = self.I(lambda: np.column_stack((self.data.Open,
+                                                                         self.data.Close)))
+
                 resampled = resample_apply('W', SMA, self.data.Close, 3)
                 resampled_ind = resample_apply('W', SMA, self.sma, 3)
                 assert np.unique(resampled[-5:]).size == 1
@@ -127,12 +131,8 @@ class TestBacktest(TestCase):
                 assert resampled in self._indicators, "Strategy.I not called"
                 assert resampled_ind in self._indicators, "Strategy.I not called"
 
-                try:
-                    self.data.X
-                except KeyError:
-                    pass
-                else:
-                    assert False
+                assert 1 == try_(lambda: self.data.X, 1, AttributeError)
+                assert 1 == try_(lambda: self.data['X'], 1, KeyError)
 
                 assert self.data.pip == .01
 
@@ -256,6 +256,7 @@ class TestBacktest(TestCase):
                 'Max. Drawdown Duration': pd.Timedelta('584 days 00:00:00'),
                 'Max. Drawdown [%]': -48.15069053929621,
                 'Max. Trade Duration': pd.Timedelta('183 days 00:00:00'),
+                'Profit Factor': 2.060450149412349,
                 'Return [%]': 426.2429346696951,
                 'SQN': 0.91553210127173,
                 'Sharpe Ratio': 0.23169782960690408,
@@ -373,6 +374,35 @@ class TestOptimize(TestCase):
         with _tempfile() as f:
             bt.plot(filename=f, open_browser=False)
 
+    @unittest.skipIf(not hasattr(_Array, 'df'), 'v2 branch is not merged')
+    def test_nowrite_df(self):
+        # Test we don't write into passed data df by default.
+        # Important for copy-on-write in Backtest.optimize()
+        df = EURUSD.astype(float)
+        values = df.values.ctypes.data
+        assert values == df.values.ctypes.data
+
+        class S(SmaCross):
+            def init(self):
+                super().init()
+                assert values == self.data.df.values.ctypes.data
+
+        bt = Backtest(df, S)
+        _ = bt.run()
+        assert values == bt._data.values.ctypes.data
+
+    def test_multiprocessing_windows_spawn(self):
+        df = GOOG.iloc[:100]
+        kw = dict(fast=[10])
+
+        stats1 = Backtest(df, SmaCross).optimize(**kw)
+        with patch('multiprocessing.get_start_method', lambda **_: 'spawn'):
+            with self.assertWarns(UserWarning) as cm:
+                stats2 = Backtest(df, SmaCross).optimize(**kw)
+
+        self.assertIn('multiprocessing support', cm.warning.args[0])
+        assert stats1.filter('[^_]').equals(stats2.filter('[^_]')), (stats1, stats2)
+
     def test_optimize_invalid_param(self):
         bt = Backtest(GOOG.iloc[:100], SmaCross)
         self.assertRaises(AttributeError, bt.optimize, foo=range(3))
@@ -531,6 +561,12 @@ class TestLib(TestCase):
         self.assertTrue((res.dropna() == res2.dropna()).all())
         self.assertTrue((res.index == res2.index).all())
 
+        res3 = resample_apply('D', None, EURUSD)
+        self.assertIn('Volume', res3)
+
+        res3 = resample_apply('D', lambda df: (df.Close, df.Close), EURUSD)
+        self.assertIsInstance(res3, pd.DataFrame)
+
     def test_plot_heatmaps(self):
         bt = Backtest(GOOG, SmaCross)
         stats, heatmap = bt.optimize(fast=range(2, 7, 2),
@@ -585,8 +621,10 @@ class TestUtil(TestCase):
         self.assertEqual(_as_str(_Indicator([1, 2], name='x')), 'x')
         self.assertEqual(_as_str(func), 'func')
         self.assertEqual(_as_str(Class), 'Class')
-        self.assertEqual(_as_str(lambda x: x), '')
-        for s in ('Open', 'High', 'Low', 'Close'):
+        self.assertEqual(_as_str(pd.Series([1, 2], name='x')), 'x')
+        self.assertEqual(_as_str(pd.DataFrame()), 'df')
+        self.assertEqual(_as_str(lambda x: x), 'λ')
+        for s in ('Open', 'High', 'Low', 'Close', 'Volume'):
             self.assertEqual(_as_str(_Array([1], name=s)), s[0])
 
 
